using System;
using System.Collections;
using Server;
using Server.Prompts;
using Server.Mobiles;
using Server.ContextMenus;
using Server.Gumps;
using Server.Items;
using Server.Network;
using Server.Targeting;
using Server.Multis;
using Server.Regions;


namespace Server.Items
{
        [FlipableAttribute( 0xE41, 0xE40 )]
        public class GraniteBox : BaseContainer
        {
                private int m_Iron;
                private int m_DullCopper;
                private int m_ShadowIron;
                private int m_Copper;
                private int m_Bronze;
                private int m_Gold;
                private int m_Agapite;
                private int m_Verite;
                private int m_Valorite;
				private int m_Sand;

                [CommandProperty( AccessLevel.GameMaster )]
                public int Iron{ get{ return m_Iron; } set{ m_Iron = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int DullCopper{ get{ return m_DullCopper; } set{ m_DullCopper = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int ShadowIron{ get{ return m_ShadowIron; } set{ m_ShadowIron = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Copper{ get{ return m_Copper; } set{ m_Copper = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Bronze{ get{ return m_Bronze; } set{ m_Bronze = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Gold{ get{ return m_Gold; } set{ m_Gold = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Agapite{ get{ return m_Agapite; } set{ m_Agapite = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Verite{ get{ return m_Verite; } set{ m_Verite = value; InvalidateProperties(); } }

                [CommandProperty( AccessLevel.GameMaster )]
                public int Valorite{ get{ return m_Valorite; } set{ m_Valorite = value; InvalidateProperties(); } }

				[CommandProperty( AccessLevel.GameMaster )]
                public int Sand{ get{ return m_Sand; } set{ m_Sand = value; InvalidateProperties(); } }

                [Constructable]
                public GraniteBox() : base( 0xE41 )
                {
                        Movable = true;
                        Weight = 100.0;
                        Hue = 0x488;
                        Name = "Granite Box";
						MaxItems = 1;
                }

                public override void OnDoubleClick( Mobile from )
                {
                        if ( Movable )
                        {
                                from.SendMessage( "You haven't locked it down!" );
                                return;
                        }
                        if ( !from.InRange( GetWorldLocation(), 2 ) )
                                from.LocalOverheadMessage( Network.MessageType.Regular, 0x3B2, 1019045 ); // I can't reach that.
                        else if ( from is PlayerMobile )
                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                }
				
				// Added by RWR 050204 to prevent items from dropping in container
				public override bool OnDragDrop ( Mobile from, Item dropped)
				{
					return false;
				}
				// End RWR

                public void BeginCombine( Mobile from )
                {
                                from.Target = new GraniteBoxTarget( this );
                }

                public void EndCombine( Mobile from, object o )
                {
                        if ( o is Item && ((Item)o).IsChildOf( from.Backpack ) )
                        {                                
                                if (!( o is BaseGranite ))
								{
									if ( o is Sand )
                                	{
                                        if ( Sand >= 500 )
                                                from.SendMessage( "The sand is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Sand = ( Sand + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
									}                                	
								else
                                {
                                        from.SendMessage( "That is not granite or sand." );
                               	}
								}
                                if ( o is DullCopperGranite )
                                {
                                        if ( DullCopper >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                DullCopper = ( DullCopper + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is ShadowIronGranite )
                                {
                                        if ( ShadowIron >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                ShadowIron = ( ShadowIron + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is CopperGranite )
                                {
                                        if ( Copper >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Copper = ( Copper + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is BronzeGranite )
                                {
                                        if ( Bronze >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Bronze = ( Bronze + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is GoldGranite )
                                {
                                        if ( Gold >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Gold = ( Gold + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is AgapiteGranite )
                                {
                                        if ( Agapite >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Agapite = ( Agapite + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is VeriteGranite )
                                {
                                        if ( Verite >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Verite = ( Verite + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is ValoriteGranite )
                                {
                                        if ( Valorite >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Valorite = ( Valorite + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
                                if ( o is Granite )
                                {
                                        if ( Iron >= 500 )
                                                from.SendMessage( "That granite type is too full to add more." );
                                        else
                                        {
                                                ((Item)o).Delete();
                                                Iron = ( Iron + 1 );
                                                from.SendGump( new GraniteBoxGump( (PlayerMobile)from, this ) );
                                                BeginCombine( from );
                                        }
                                }
								
                        }
                        else
                        {
                                from.SendLocalizedMessage( 1045158 ); // You must have the item in your backpack to target it.
                        }
                }

                public GraniteBox( Serial serial ) : base( serial )
                {
                }

                                public override void Serialize( GenericWriter writer ) 
		                { 
		                        base.Serialize( writer ); 
		
		                        writer.Write( (int) m_Iron); 
		                        writer.Write( (int) m_DullCopper); 
		                        writer.Write( (int) m_ShadowIron); 
		                        writer.Write( (int) m_Copper); 
		                        writer.Write( (int) m_Bronze); 
		                        writer.Write( (int) m_Gold); 
		                        writer.Write( (int) m_Agapite); 
		                        writer.Write( (int) m_Verite); 
		                        writer.Write( (int) m_Valorite);
								writer.Write( (int) m_Sand);
		
		                } 
		
		                public override void Deserialize( GenericReader reader ) 
		                { 
		                        base.Deserialize( reader ); 
		
		                        m_Iron = reader.ReadInt(); 
		                        m_DullCopper = reader.ReadInt(); 
		                        m_ShadowIron = reader.ReadInt(); 
		                        m_Copper = reader.ReadInt(); 
		                        m_Bronze = reader.ReadInt(); 
		                        m_Gold = reader.ReadInt(); 
		                        m_Agapite = reader.ReadInt(); 
		                        m_Verite = reader.ReadInt(); 
		                        m_Valorite = reader.ReadInt();
								m_Sand = reader.ReadInt();
		                } 
		        } 
		} 
		

namespace Server.Items
{
        public class GraniteBoxGump : Gump
        {
                private PlayerMobile m_From;
                private GraniteBox m_Box;

                public GraniteBoxGump( PlayerMobile from, GraniteBox box ) : base( 25, 25 )
                {
                        m_From = from;
                        m_Box = box;

                        m_From.CloseGump( typeof( GraniteBoxGump ) );

                        AddPage( 0 );

                        AddBackground( 50, 10, 455, 260, 5054 );
                        AddImageTiled( 58, 20, 438, 241, 2624 );
                        AddAlphaRegion( 58, 20, 438, 241 );

                        AddLabel( 225, 25, 0x480, "Granite Box" );

                        AddLabel( 125, 50, 0x486, "Iron" );
                        AddLabel( 225, 50, 0x480, box.Iron.ToString() );
                        AddButton( 75, 50, 4005, 4007, 1, GumpButtonType.Reply, 0 );
                        AddLabel( 125, 75, 0x486, "Dull Copper" );
                        AddLabel( 225, 75, 0x480, box.DullCopper.ToString() );
                        AddButton( 75, 75, 4005, 4007, 2, GumpButtonType.Reply, 0 );
                        AddLabel( 125, 100, 0x486, "Shadow Iron" );
                        AddLabel( 225, 100, 0x480, box.ShadowIron.ToString() );
                        AddButton( 75, 100, 4005, 4007, 3, GumpButtonType.Reply, 0 );
                        AddLabel( 125, 125, 0x486, "Copper" );
                        AddLabel( 225, 125, 0x480, box.Copper.ToString() );
                        AddButton( 75, 125, 4005, 4007, 4, GumpButtonType.Reply, 0 );
                        AddLabel( 125, 150, 0x486, "Bronze" );
                        AddLabel( 225, 150, 0x480, box.Bronze.ToString() );
                        AddButton( 75, 150, 4005, 4007, 5, GumpButtonType.Reply, 0 );
                        AddLabel( 325, 50, 0x486, "Gold" );
                        AddLabel( 425, 50, 0x480, box.Gold.ToString() );
                        AddButton( 275, 50, 4005, 4007, 6, GumpButtonType.Reply, 0 );
                        AddLabel( 325, 75, 0x486, "Agapite" );
                        AddLabel( 425, 75, 0x480, box.Agapite.ToString() );
                        AddButton( 275, 75, 4005, 4007, 7, GumpButtonType.Reply, 0 );
                        AddLabel( 325, 100, 0x486, "Verite" );
                        AddLabel( 425, 100, 0x480, box.Verite.ToString() );
                        AddButton( 275, 100, 4005, 4007, 8, GumpButtonType.Reply, 0 );
                        AddLabel( 325, 125, 0x486, "Valorite" );
                        AddLabel( 425, 125, 0x480, box.Valorite.ToString() );
                        AddButton( 275, 125, 4005, 4007, 9, GumpButtonType.Reply, 0 );
						AddLabel( 325, 150, 0x486, "Sand" );
                        AddLabel( 425, 150, 0x480, box.Sand.ToString() );
                        AddButton( 275, 150, 4005, 4007, 10, GumpButtonType.Reply, 0 );
                        
                        AddButton( 275, 200, 4005, 4007, 11, GumpButtonType.Reply, 0 );
                        AddLabel( 325, 200, 0x486, "Add Granite" );
                        
                }

                public override void OnResponse( NetState sender, RelayInfo info )
                {
                        if ( m_Box.Deleted )
                                return;

                        if ( info.ButtonID == 1 )
                        {
                                if ( m_Box.Iron > 0 )
                                {
                                        m_Box.Iron = ( m_Box.Iron - 1 );
                                        m_From.AddToBackpack( new Granite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 2 )
                        {
                                if ( m_Box.DullCopper > 0 )
                                {
                                        m_Box.DullCopper = ( m_Box.DullCopper - 1 );
                                        m_From.AddToBackpack( new DullCopperGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 3 )
                        {
                                if ( m_Box.ShadowIron > 0 )
                                {
                                        m_Box.ShadowIron = ( m_Box.ShadowIron - 1 );
                                        m_From.AddToBackpack( new ShadowIronGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 4 )
                        {
                                if ( m_Box.Copper > 0 )
                                {
                                        m_Box.Copper = ( m_Box.Copper - 1 );
                                        m_From.AddToBackpack( new CopperGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 5 )
                        {
                                if ( m_Box.Bronze > 0 )
                                {
                                        m_Box.Bronze = ( m_Box.Bronze - 1 );
                                        m_From.AddToBackpack( new BronzeGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 6 )
                        {
                                if ( m_Box.Gold > 0 )
                                {
                                        m_Box.Gold = ( m_Box.Gold - 1 );
                                        m_From.AddToBackpack( new GoldGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 7 )
                        {
                                if ( m_Box.Agapite > 0 )
                                {
                                        m_Box.Agapite = ( m_Box.Agapite - 1 );
                                        m_From.AddToBackpack( new AgapiteGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 8 )
                        {
                                if ( m_Box.Verite > 0 )
                                {
                                        m_Box.Verite = ( m_Box.Verite - 1 );
                                        m_From.AddToBackpack( new VeriteGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 9 )
                        {
                                if ( m_Box.Valorite > 0 )
                                {
                                        m_Box.Valorite = ( m_Box.Valorite - 1 );
                                        m_From.AddToBackpack( new ValoriteGranite() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any of that granite!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
						if ( info.ButtonID == 10 )
                        {
                                if ( m_Box.Sand > 0 )
                                {
                                        m_Box.Sand = ( m_Box.Sand - 1 );
                                        m_From.AddToBackpack( new Sand() );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                                else
                                {
                                        m_From.SendMessage( "You do not have any sand!" );
                                        m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                }
                        }
                        if ( info.ButtonID == 11 )
                        {
                                m_From.SendGump( new GraniteBoxGump( m_From, m_Box ) );
                                m_Box.BeginCombine( m_From );
                        }
                }
        }

}

namespace Server.Items
{
        public class GraniteBoxTarget : Target
        {
                private GraniteBox m_Box;

                public GraniteBoxTarget( GraniteBox box ) : base( 18, false, TargetFlags.None )
                {
                        m_Box = box;
                }

                protected override void OnTarget( Mobile from, object targeted )
                {
                        if ( m_Box.Deleted )
                                return;

                        m_Box.EndCombine( from, targeted );
                }
        }
}
